# -*- coding: utf-8 -*-
"""
    vollib.black.greeks.analytical
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    A library for option pricing, implied volatility, and
    greek calculation.  vollib is based on lets_be_rational,
    a Python wrapper for LetsBeRational by Peter Jaeckel as 
    described below.

    :copyright: © 2015 Iota Technologies Pte Ltd    
    :license: MIT, see LICENSE for more details.

    About LetsBeRational:
    ~~~~~~~~~~~~~~~~~~~~~~~

    The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

    ::

      ======================================================================================
      Copyright © 2013-2014 Peter Jäckel.

      Permission to use, copy, modify, and distribute this software is freely granted,
      provided that this notice is preserved.

      WARRANTY DISCLAIMER
      The Software is provided "as is" without warranty of any kind, either express or implied,
      including without limitation any implied warranties of condition, uninterrupted use,
      merchantability, fitness for a particular purpose, or non-infringement.
      ======================================================================================
"""

# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports

# Related third party imports
import numpy

# Local application/library specific imports
from lets_be_rational import norm_cdf as cnd
from vollib.helper import pdf
from vollib.black import d1,d2, black

# -----------------------------------------------------------------------------
# FUNCTIONS - ANALYTICAL GREEKS


def delta(flag, F, K, t, r, sigma):
    
    
    """Returns the Black delta of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float

    :returns:  float 
      
    
    >>> F = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> v1 = delta(flag, F, K, t, r, sigma)
    >>> v2 = 0.45107017482201828
    >>> abs(v1-v2) < .000001
    True
    """

    D1 = d1(F, K, t, r, sigma)

    if flag == 'p':
        return - numpy.exp(-r*t) * cnd(-D1)
    else:
        return numpy.exp(-r*t) * cnd(D1)


def theta(flag, F, K, t, r, sigma):


    """Returns the Black theta of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float

    :returns:  float 

    >>> F = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> v1 = theta(flag, F, K, t, r, sigma)
    >>> v2 = -0.00816236877462
    >>> abs(v1-v2) < .000001
    True
    >>> flag = 'p'
    >>> v1 = theta(flag, F, K, t, r, sigma)
    >>> v2 = -0.00802799155312
    >>> abs(v1-v2) < .000001
    True
    """

    e_to_the_minus_rt = numpy.exp(-r*t)
    two_sqrt_t = 2 * numpy.sqrt(t)

    D1 = d1(F, K, t, r, sigma)
    D2 = d2(F, K, t, r, sigma)
    pdf_d1 = pdf(D1)
    cnd_d2 = cnd(D2)

    first_term = F * e_to_the_minus_rt * pdf(D1) * sigma / two_sqrt_t 

    if flag == 'c':        
        second_term = -r * F * e_to_the_minus_rt * cnd(D1) 
        third_term = r * K * e_to_the_minus_rt * cnd(D2)
        return -(first_term + second_term + third_term) / 365.
    else:
        second_term = -r * F * e_to_the_minus_rt * cnd(-D1)
        third_term = r * K * e_to_the_minus_rt * cnd(-D2)
        return (-first_term + second_term + third_term) / 365.0
    
    return (first_term - second_term)/365.0



def gamma(flag, F, K, t, r, sigma):
    
    """Returns the Black gamma of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float

    :returns:  float 

    >>> F = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> v1 = gamma(flag, F, K, t, r, sigma)
    >>> # 0.0640646705882
    >>> v2 = 0.0640646705882
    >>> abs(v1-v2) < .000001
    True
    """
    

    D1 = d1(F, K, t, r, sigma)
    return pdf(D1)*numpy.exp(-r*t)/(F*sigma*numpy.sqrt(t))


def vega(flag, F, K, t, r, sigma):


    """Returns the Black vega of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float

    :returns:  float     
    
    ::
    
      ==========================================================
      Note: The text book analytical formula does not multiply by .01,
      but in practice vega is defined as the change in price
      for each 1 percent change in IV, hence we multiply by 0.01.
      ==========================================================
    
    
    >>> F = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> v1 = vega(flag, F, K, t, r, sigma)
    >>> # 0.118317785624
    >>> v2 = 0.118317785624
    >>> abs(v1-v2) < .000001
    True

    """

    D1 = d1(F, K, t, r, sigma)
    return F * numpy.exp(-r*t) * pdf(D1) * numpy.sqrt(t) * 0.01


def rho(flag, F, K, t, r, sigma):
    
    """Returns the Black rho of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float

    :returns:  float     
    
    
    ::

      ==========================================================
      The text book analytical formula does not multiply by .01,
      but in practice rho is defined as the change in price
      for each 1 percent change in r, hence we multiply by 0.01.
      ==========================================================
      
    >>> F = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> v1 = rho(flag, F, K, t, r, sigma)
    >>> v2 = -0.0074705380059582258
    >>> abs(v1-v2) < .000001
    True
    >>> flag = 'p'
    >>> v1 = rho(flag, F, K, t, r, sigma)
    >>> v2 = -0.011243286001308292
    >>> abs(v1-v2) < .000001
    True
    """

    return -t * black(flag, F, K, t, r, sigma) * .01


# -----------------------------------------------------------------------------
# MAIN
if __name__=='__main__':  
    import doctest
    if not doctest.testmod().failed:
        print "Doctest passed"






# -*- coding: utf-8 -*-
"""
    vollib.black_scholes
    ~~~~~~~~~~~~~~~~~~~~

    A library for option pricing, implied volatility, and
    greek calculation.  vollib is based on lets_be_rational,
    a Python wrapper for LetsBeRational by Peter Jaeckel as 
    described below.

    :copyright: © 2015 Iota Technologies Pte Ltd    
    :license: MIT, see LICENSE for more details.

    About LetsBeRational:
    ~~~~~~~~~~~~~~~~~~~~~~~

    The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

    :: 

      ======================================================================================
      Copyright © 2013-2014 Peter Jäckel.

      Permission to use, copy, modify, and distribute this software is freely granted,
      provided that this notice is preserved.

      WARRANTY DISCLAIMER
      The Software is provided "as is" without warranty of any kind, either express or implied,
      including without limitation any implied warranties of condition, uninterrupted use,
      merchantability, fitness for a particular purpose, or non-infringement.
      ======================================================================================

"""

# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports
from math import e

# Related third party imports
import numpy

# Local application/library specific imports
from vollib.helper import forward_price
from vollib.black import black as vollib_black
from vollib.black import undiscounted_black
from vollib.helper import pdf
from lets_be_rational import norm_cdf as cnd

# -----------------------------------------------------------------------------
# FUNCTIONS - REFERENCE PYTHON IMPLEMENTATION, FOR COMPARISON


def d1(S,K,t,r,sigma):  # see Hull, page 292
    
    """Calculate the d1 component of the Black-Scholes PDE.

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float    
    
    
    John C. Hull, "Options, Futures and Other Derivatives," 
    7th edition, Example 13.6, page 294
    
    >>> S = 42
    >>> K = 40
    >>> r = .10
    >>> sigma = .20
    >>> t = 0.5
    >>> calculated_d1 = d1(S,K,t,r,sigma)
    >>> text_book_d1 = 0.7693
    >>> abs(calculated_d1 - text_book_d1) < 0.0001
    True
    """

    sigma_squared = sigma*sigma
    numerator = numpy.log( S/float(K) ) + ( r + sigma_squared/2.) * t
    denominator = sigma * numpy.sqrt(t)

    return numerator/denominator

def d2(S,K,t,r,sigma):  # see Hull, page 292

    """Calculate the d2 component of the Black-Scholes PDE.

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float  
        
    John C. Hull, "Options, Futures and Other Derivatives," 
    7th edition, Example 13.6, page 294

    >>> S = 42
    >>> K = 40
    >>> r = .10
    >>> sigma = .20
    >>> t = 0.5
    >>> calculated_d2 = d2(S,K,t,r,sigma) #0.627841271869
    >>> text_book_d2 = 0.6278
    >>> abs(calculated_d2 - text_book_d2) < 0.0001
    True
    """
    
    return d1(S, K, t, r, sigma) - sigma*numpy.sqrt(t)

def python_black_scholes(flag, S, K, t, r, sigma):
    
    """Return the Black-Scholes option price implemented in
        python (for reference).

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param flag: 'c' or 'p' for call or put.
    :type flag: str    
    
    >>> S,K,t,r,sigma = 60,65,.25,.08,.3
    >>> c1 = black_scholes('c',S,K,t,r,sigma) 
    >>> c2 = python_black_scholes('c',S,K,t,r,sigma)    
    >>> abs(c1 - c2) < .00001
    True
    >>> abs(c1 - 2.13336844492) < .00001
    True
    """
    
    e_to_the_minus_rt = numpy.exp(-r*t)
    D1 = d1(S, K, t, r, sigma)
    D2 = d2(S, K, t, r, sigma)
    if flag == 'c':
        return S * cnd(D1) - K * e_to_the_minus_rt * cnd(D2)
    else:
        return - S * cnd(-D1) + K * e_to_the_minus_rt * cnd(-D2)

# -----------------------------------------------------------------------------
# FUNCTIONS 

def black_scholes(flag, S, K, t, r, sigma): 

    """Return the Black-Scholes option price.

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param flag: 'c' or 'p' for call or put.
    :type flag: str    
    
    
    >>> c = black_scholes('c',100,90,.5,.01,.2) 
    >>> abs(c - 12.111581435) < .000001
    True

    >>> p = black_scholes('p',100,90,.5,.01,.2) 
    >>> abs(p - 1.66270456231) < .000001
    True
    
    >>> flag, S, K, t, r, sigma = 'c',100,90,.5,.01,.2
    >>> c = black_scholes(flag, S, K, t, r, sigma) 
    >>> python_c = python_black_scholes(flag, S, K, t, r, sigma)
    >>> abs(c - python_c) < .000001
    True


    >>> flag, S, K, t, r, sigma = 'p',100,90,.5,.01,.2
    >>> p = black_scholes(flag, S, K, t, r, sigma) 
    >>> python_p = python_black_scholes(flag, S, K, t, r, sigma)
    >>> abs(p - python_p) < .000001
    True
    
    """   
    
    discount_factor = numpy.exp(-r*t)
    F = S / discount_factor
    return undiscounted_black(F, K, sigma, t, flag) * discount_factor

# -----------------------------------------------------------------------------
# MAIN
if __name__=='__main__':  
    import doctest
    if not doctest.testmod().failed:
        print "Doctest passed"



# -*- coding: utf-8 -*-
"""
    vollib.black_scholes.greeks.analytical
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    A library for option pricing, implied volatility, and
    greek calculation.  vollib is based on lets_be_rational,
    a Python wrapper for LetsBeRational by Peter Jaeckel as 
    described below.

    :copyright: © 2015 Iota Technologies Pte Ltd    
    :license: MIT, see LICENSE for more details.

    About LetsBeRational:
    ~~~~~~~~~~~~~~~~~~~~~~~

    The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

    ::

      ======================================================================================
      Copyright © 2013-2014 Peter Jäckel.

      Permission to use, copy, modify, and distribute this software is freely granted,
      provided that this notice is preserved.

      WARRANTY DISCLAIMER
      The Software is provided "as is" without warranty of any kind, either express or implied,
      including without limitation any implied warranties of condition, uninterrupted use,
      merchantability, fitness for a particular purpose, or non-infringement.
      ======================================================================================

"""


# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports


# Related third party imports
import numpy

# Local application/library specific imports
from lets_be_rational import norm_cdf as cnd
from vollib.helper import pdf
from vollib.black_scholes import d1,d2

# -----------------------------------------------------------------------------
# FUNCTIONS - ANALYTICAL GREEKS


def delta(flag, S, K, t, r, sigma):
    
    """Return Black-Scholes delta of an option.
    
    
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param flag: 'c' or 'p' for call or put.
    :type flag: str      
    
    
    Example 17.1, page 355, Hull:
    
    >>> S = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> delta_calc = delta(flag, S, K, t, r, sigma)
    >>> # 0.521601633972
    >>> delta_text_book = 0.522
    >>> abs(delta_calc - delta_text_book) < .01
    True
    """

    d_1 = d1(S, K, t, r, sigma)

    if flag == 'p':
        return cnd(d_1) - 1.0
    else:
        return cnd(d_1)


def theta(flag, S, K, t, r, sigma):

    """Return Black-Scholes theta of an option.
    
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param flag: 'c' or 'p' for call or put.
    :type flag: str      
    
    
    The text book analytical formula does not divide by 365,
    but in practice theta is defined as the change in price
    for each day change in t, hence we divide by 365.

    Example 17.2, page 359, Hull:

    >>> S = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> annual_theta_calc = theta(flag, S, K, t, r, sigma) * 365
    >>> # -4.30538996455
    >>> annual_theta_text_book = -4.31
    >>> abs(annual_theta_calc - annual_theta_text_book) < .01
    True
    """

    e_to_the_minus_rt = numpy.exp(-r*t)
    two_sqrt_t = 2 * numpy.sqrt(t)

    d_1 = d1(S, K, t, r, sigma)
    d_2 = d2(S, K, t, r, sigma)
    pdf_d1 = pdf(d_1)
    cnd_d2 = cnd(d_2)

    first_term = (-S * pdf_d1 * sigma) / two_sqrt_t 
    second_term = r * K * e_to_the_minus_rt * cnd_d2

    return (first_term - second_term)/365.0



def gamma(flag, S, K, t, r, sigma):
    
    """Return Black-Scholes gamma of an option.
    
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param flag: 'c' or 'p' for call or put.
    :type flag: str      
    
    Example 17.4, page 364, Hull:

    >>> S = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> gamma_calc = gamma(flag, S, K, t, r, sigma)
    >>> # 0.0655453772525
    >>> gamma_text_book = 0.066
    >>> abs(gamma_calc - gamma_text_book) < .001
    True
    """
    

    d_1 = d1(S, K, t, r, sigma)
    v_squared = sigma**2
    return pdf(d_1)/(S*sigma*numpy.sqrt(t))


def vega(flag, S, K, t, r, sigma):

    """Return Black-Scholes vega of an option.
    
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param flag: 'c' or 'p' for call or put.
    :type flag: str      
    
    
    The text book analytical formula does not multiply by .01,
    but in practice vega is defined as the change in price
    for each 1 percent change in IV, hence we multiply by 0.01.


    Example 17.6, page 367, Hull:
    
    >>> S = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> vega_calc = vega(flag, S, K, t, r, sigma)
    >>> # 0.121052427542
    >>> vega_text_book = 0.121
    >>> abs(vega_calc - vega_text_book) < .01
    True

    """

    d_1 = d1(S, K, t, r, sigma)
    return S * pdf(d_1) * numpy.sqrt(t) * 0.01


def rho(flag, S, K, t, r, sigma):

    """Return Black-Scholes rho of an option.
    
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param flag: 'c' or 'p' for call or put.
    :type flag: str      
    
    The text book analytical formula does not multiply by .01,
    but in practice rho is defined as the change in price
    for each 1 percent change in r, hence we multiply by 0.01.


    Example 17.7, page 368, Hull:

    >>> S = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> rho_calc = rho(flag, S, K, t, r, sigma)
    >>> # 0.089065740988
    >>> rho_text_book = 0.0891
    >>> abs(rho_calc - rho_text_book) < .0001
    True

    """

    d_2 = d2(S, K, t, r, sigma)
    e_to_the_minus_rt = numpy.exp(-r*t)
    if flag == 'c':
        return t*K*e_to_the_minus_rt * cnd(d_2) * .01
    else:
        return -t*K*e_to_the_minus_rt * cnd(-d_2) * .01


# -----------------------------------------------------------------------------
# MAIN
if __name__=='__main__':  
    import doctest
    if not doctest.testmod().failed:
        print "Doctest passed"



# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports

# Related third party imports
import numpy
from matplotlib import pyplot as plt
from scipy.optimize import brent

# Local application/library specific imports
from vollib.black import black

# -----------------------------------------------------------------------------
# FUNCTIONS - IMPLIED UNDERLYING PRICE AND INTEREST RATE

def find_r_and_F(K,C,P,t):
    
    """
    We assume that the interest rate r and futures price F
    are unknown, and all we know is are the option prices
    and the time till expiration t.
    We seek r as the value which minimizes the standard 
    deviation of the resultant futures prices, calculated
    per strike as F = (C-P)/e^-rt + K.
    
    >>> K,C,P,t = make_test_data(100, .2, .015, .5, 0.0)
    >>> r, F = find_r_and_F(K, C, P, t)
    >>> abs(r - .015) <.000001
    True
    >>> abs(F - 100) <.000001
    True
    
    """

    def function_to_minimize(r):
        e_to_the_minus_rt = numpy.exp(-r*t)
        F = []
        for i in range(len(K)):
            F.append(
                C[i]/e_to_the_minus_rt - P[i]/e_to_the_minus_rt + K[i]
            )
        return numpy.std(F)

    r = brent(function_to_minimize)
    e_to_the_minus_rt = numpy.exp(-r*t)
    F =[]
    for i in range(len(K)):
        F.append(
            C[i]/e_to_the_minus_rt - P[i]/e_to_the_minus_rt + K[i]
        )
    F_mean = numpy.average(F)
    return r, F_mean

# -----------------------------------------------------------------------------
# FUNCTIONS - TEST

def make_test_data(F, sigma, r, t, noise):
    
    '''
    Returns a test option chain with up to 100 strikes 
    surrounding the futures price F.
    '''
    
    K= range(max(1,F-50),F+50)
    
    calls, puts = [],[]

    for Ki in K:
        P, C = black(F, Ki, sigma, r, t, 'p'), black(F, Ki, sigma,r, t,'c')
        P += numpy.random.rand() * noise * numpy.random.choice([-1,1])
        C += numpy.random.rand() * noise * numpy.random.choice([-1,1])
        calls.append(C)
        puts.append(P)

    return K, calls, puts, t


def plot_error():
    r = .03
    F = 100
    noise = numpy.linspace(0,2,1000)
    r_error, F_error = [],[]
    for n in noise:
        K, C, P, t = make_test_data(
            F=F, sigma=0.3, r=r, t=.5, noise=n)
        r_estimate, F_estimate = find_r_and_F(K, C, P, t)
        r_error.append(r_estimate-r)
        F_error.append(F_estimate-F)
    
    plt.title('Option Implied Futures Price')
    plt.plot(noise,r_error,label = 'r error', color = 'red')
    plt.plot(noise,F_error, label = 'F error',alpha = 0.6)
    plt.legend(loc = 'best')
    plt.grid()
    plt.ylim(-.2,.2)
    plt.xlabel('noise amount')
    plt.ylabel('error')
    plt.show()
plot_error()

# -----------------------------------------------------------------------------
# MAIN
if __name__=='__main__':
    import doctest
    if not doctest.testmod().failed:
        print "Doctest passed"



# -*- coding: utf-8 -*-
"""
    vollib.black_scholes_merton
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    A library for option pricing, implied volatility, and
    greek calculation.  vollib is based on lets_be_rational,
    a Python wrapper for LetsBeRational by Peter Jaeckel as 
    described below.

    :copyright: © 2015 Iota Technologies Pte Ltd    
    :license: MIT, see LICENSE for more details.

    About LetsBeRational:
    ~~~~~~~~~~~~~~~~~~~~~~~

    The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

    ::

      ======================================================================================
      Copyright © 2013-2014 Peter Jäckel.

      Permission to use, copy, modify, and distribute this software is freely granted,
      provided that this notice is preserved.

      WARRANTY DISCLAIMER
      The Software is provided "as is" without warranty of any kind, either express or implied,
      including without limitation any implied warranties of condition, uninterrupted use,
      merchantability, fitness for a particular purpose, or non-infringement.
      ======================================================================================

"""

# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports
import numpy

# Related third party imports


# Local application/library specific imports
from lets_be_rational import black
from lets_be_rational import norm_cdf as cnd
from vollib.helper import forward_price
from vollib.helper import binary_flag
from vollib.helper import pdf

# -----------------------------------------------------------------------------
# FUNCTIONS, FOR REFERENCE AND TESTING

def d1(S, K, t, r, sigma, q):
    
    """Calculate the d1 component of the Black-Scholes-Merton PDE.

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param q: annualized continuous dividend rate
    :type q: float  
    
    
    From Espen Haug, The Complete Guide To Option Pricing Formulas
    Page 4

    >>> S=100
    >>> K=95
    >>> q=.05
    >>> t = 0.5
    >>> r = 0.1
    >>> sigma = 0.2
    
    >>> d1_published_value = 0.6102
    >>> d1_calc = d1(S,K,t,r,sigma,q)
    >>> abs(d1_published_value - d1_calc) < 0.0001
    True
    
    """
    
    numerator = numpy.log(S/float(K)) + (r - q + sigma*sigma/2.0)*t
    denominator = sigma * numpy.sqrt(t)
    return numerator/denominator
    
    
def d2(S, K, t, r, sigma, q):
    
    """Calculate the d2 component of the Black-Scholes-Merton PDE.

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param q: annualized continuous dividend rate
    :type q: float
    

    From Espen Haug, The Complete Guide To Option Pricing Formulas
    Page 4

    >>> S=100
    >>> K=95
    >>> q=.05
    >>> t = 0.5
    >>> r = 0.1
    >>> sigma = 0.2

    >>> d2_published_value = 0.4688
    >>> d2_calc = d2(S,K,t,r,sigma,q)
    >>> abs(d2_published_value - d2_calc) < 0.0001
    True

    """    
    
    return d1(S, K, t, r, sigma, q) - sigma*numpy.sqrt(t)


def bsm_call(S, K, t, r, sigma, q):

    """Return the Black-Scholes-Merton call price
    implemented in python (for reference).
    
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param q: annualized continuous dividend rate
    :type q: float 
    """
   
    D1 = d1(S, K, t, r, sigma, q)
    D2 = d2(S, K, t, r, sigma, q)    
    
    return S * numpy.exp(-q*t) * cnd(D1) - K * numpy.exp(-r*t) * cnd(D2)


def bsm_put(S, K, t, r, sigma, q):

    
    """Return the Black-Scholes-Merton put price
    implemented in python (for reference).

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param q: annualized continuous dividend rate
    :type q: float 
  
    
    
    From Espen Haug, The Complete Guide To Option Pricing Formulas
    Page 4

    >>> S=100
    >>> K=95
    >>> q=.05
    >>> t = 0.5
    >>> r = 0.1
    >>> sigma = 0.2

    >>> p_published_value = 2.4648
    >>> p_calc = bsm_put(S, K, t, r, sigma, q)
    >>> abs(p_published_value - p_calc) < 0.0001
    True

    """     
    
    D1 = d1(S, K, t, r, sigma, q)
    D2 = d2(S, K, t, r, sigma, q)
    return K * numpy.exp(-r*t) * cnd(-D2) - S * numpy.exp(-q*t) * cnd(-D1)



def python_black_scholes_merton(flag, S, K, t, r, sigma, q):
    
    """Return the Black-Scholes-Merton call price implemented in
    python (for reference).

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param q: annualized continuous dividend rate
    :type q: float 
    :param flag: 'c' or 'p' for call or put.
    :type flag: str

    From Espen Haug, The Complete Guide To Option Pricing Formulas
    Page 4

    >>> S=100
    >>> K=95
    >>> q=.05
    >>> t = 0.5
    >>> r = 0.1
    >>> sigma = 0.2

    >>> p_published_value = 2.4648
    >>> p_calc = python_black_scholes_merton('p', S, K, t, r, sigma, q)
    >>> abs(p_published_value - p_calc) < 0.0001
    True

    >>> c1 = python_black_scholes_merton('c', S, K, t, r, sigma, q)
    >>> c2 = black_scholes_merton('c', S, K, t, r, sigma, q)
    >>> abs(c1-c2) < .0001
    True
    
    >>> p1 = python_black_scholes_merton('p', S, K, t, r, sigma, q)
    >>> p2 = black_scholes_merton('p', S, K, t, r, sigma, q)
    >>> abs(p1-p2) < .0001
    True
    
    """    
    
    if flag == 'c':
        return bsm_call(S, K, t, r, sigma, q)
    else:
        return bsm_put(S, K, t, r, sigma, q)




def black_scholes_merton(flag, S, K, t, r, sigma, q):

    """Return the Black-Scholes-Merton option price.

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param q: annualized continuous dividend rate
    :type q: float 

    From Espen Haug, The Complete Guide To Option Pricing Formulas
    Page 4

    >>> S=100
    >>> K=95
    >>> q=.05
    >>> t = 0.5
    >>> r = 0.1
    >>> sigma = 0.2

    >>> p_published_value = 2.4648
    >>> p_calc = black_scholes_merton('p', S, K, t, r, sigma, q)
    >>> abs(p_published_value - p_calc) < 0.0001
    True
    
    >>> c1 = bsm_call(S, K, t, r, sigma, q)
    >>> c2 = black_scholes_merton('c', S, K, t, r, sigma, q)
    >>> abs(c1-c2) < .0001
    True
    """
    
    S = S * numpy.exp((r-q)*t)
    p = black(S, K, sigma, t, binary_flag[flag])
    conversion_factor = numpy.exp(-r*t)
    return p * conversion_factor


# -----------------------------------------------------------------------------
# MAIN
if __name__=='__main__':
    import doctest
    if not doctest.testmod().failed:
        print "Doctest passed"



# -*- coding: utf-8 -*-
"""
    vollib.helper.numerical_greeks
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    A library for option pricing, implied volatility, and
    greek calculation.  vollib is based on lets_be_rational,
    a Python wrapper for LetsBeRational by Peter Jaeckel as 
    described below.

    :copyright: © 2015 Iota Technologies Pte Ltd    
    :license: MIT, see LICENSE for more details.

    About LetsBeRational:
    ~~~~~~~~~~~~~~~~~~~~~~~

    The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

    ::

      ======================================================================================
      Copyright © 2013-2014 Peter Jäckel.

      Permission to use, copy, modify, and distribute this software is freely granted,
      provided that this notice is preserved.

      WARRANTY DISCLAIMER
      The Software is provided "as is" without warranty of any kind, either express or implied,
      including without limitation any implied warranties of condition, uninterrupted use,
      merchantability, fitness for a particular purpose, or non-infringement.
      ======================================================================================

    Note about the parameter "b":
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    ::

      ======================================================================================
       from Espen Gaarder Haug's 
      "The Complete Guide to Option Pricing Formulas," Second Edition, 
      page 90.

      +-----------+------------------------------------------------------+
      | b = r     |  gives the Black and Scholes (1973) stock option     |
      |           |  model                                               |
      +-----------+------------------------------------------------------+
      | b = r -q  |  gives the Merton (1973) stock option model with     |
      |           |  continuous dividend yield q                         |
      +-----------+------------------------------------------------------+
      | b = 0     |  gives the Black (1976) futures option model         | 
      +-----------+------------------------------------------------------+
      | b = 0 and |  gives the Asay (1982) margined futures option model |
      | r = 0     |                                                      |
      +-----------+------------------------------------------------------+
      ======================================================================================

"""


# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports

# Related third party imports

# Local application/library specific imports


# -----------------------------------------------------------------------------
# FUNCTIONS - GENERIC FUNCTIONS FOR NUMERICAL GREEK CALCULATION

dS = .01

def delta(flag, S, K, t, r, sigma, b, pricing_function):

    """Calculate option delta using numerical integration. 

        :param S: underlying asset price
        :type S: float
        :param K: strike price
        :type K: float
        :param sigma: annualized standard deviation, or volatility
        :type sigma: float
        :param t: time to expiration in years
        :type t: float
        :param r: risk-free interest rate
        :type r: float
        :param b: see above
        :type b: float 
        :param flag: 'c' or 'p' for call or put.
        :type flag: str
        :param pricing_function: any function returning the price of an option
        :type pricing_function: python function object

    """
    if t == 0.0:
        if S == K:
            return {'c':0.5, 'p':-0.5}[flag]
        elif S > K:
            return {'c':1.0, 'p':0.0}[flag]
        else:
            return {'c':0.0, 'p':-1.0}[flag]
    else:
        return (pricing_function(flag, S + dS, K, t, r, sigma, b) - \
                pricing_function(flag, S - dS, K, t, r, sigma, b)) / (2 * dS)

def theta(flag, S, K, t, r, sigma, b, pricing_function):

    """Calculate option theta using numerical integration. 

        :param S: underlying asset price
        :type S: float
        :param K: strike price
        :type K: float
        :param sigma: annualized standard deviation, or volatility
        :type sigma: float
        :param t: time to expiration in years
        :type t: float
        :param r: risk-free interest rate
        :type r: float
        :param b: see above
        :type b: float 
        :param flag: 'c' or 'p' for call or put.
        :type flag: str
        :param pricing_function: any function returning the price of an option
        :type pricing_function: python function object

    """



    if t <= 1. / 365.:
        return pricing_function(flag, S, K, 0.00001, r, sigma, b) - \
               pricing_function(flag, S, K, t, r, sigma, b)
    else:
        return pricing_function(flag, S, K, t - 1. / 365., r, sigma, b) - \
               pricing_function(flag, S, K, t, r, sigma, b)

def vega(flag, S, K, t, r, sigma, b, pricing_function):

    """Calculate option vega using numerical integration. 

        :param S: underlying asset price
        :type S: float
        :param K: strike price
        :type K: float
        :param sigma: annualized standard deviation, or volatility
        :type sigma: float
        :param t: time to expiration in years
        :type t: float
        :param r: risk-free interest rate
        :type r: float
        :param b: see above
        :type b: float 
        :param flag: 'c' or 'p' for call or put.
        :type flag: str
        :param pricing_function: any function returning the price of an option
        :type pricing_function: python function object

    """


    return (pricing_function(flag, S, K, t, r, sigma + 0.01, b) - \
            pricing_function(flag, S, K, t, r, sigma - 0.01, b)) / 2.

def rho(flag, S, K, t, r, sigma, b, pricing_function):

    """Calculate option rho using numerical integration. 

        :param S: underlying asset price
        :type S: float
        :param K: strike price
        :type K: float
        :param sigma: annualized standard deviation, or volatility
        :type sigma: float
        :param t: time to expiration in years
        :type t: float
        :param r: risk-free interest rate
        :type r: float
        :param b: see above
        :type b: float 
        :param flag: 'c' or 'p' for call or put.
        :type flag: str
        :param pricing_function: any function returning the price of an option
        :type pricing_function: python function object

    """


    return (pricing_function(flag, S, K, t, r + 0.01, sigma,  b ) - \
            pricing_function(flag, S, K, t, r - 0.01, sigma, b )) / (2)

def gamma(flag, S, K, t, r, sigma, b, pricing_function):


    """Calculate option gamma using numerical integration. 

        :param S: underlying asset price
        :type S: float
        :param K: strike price
        :type K: float
        :param sigma: annualized standard deviation, or volatility
        :type sigma: float
        :param t: time to expiration in years
        :type t: float
        :param r: risk-free interest rate
        :type r: float
        :param b: see above
        :type b: float 
        :param flag: 'c' or 'p' for call or put.
        :type flag: str
        :param pricing_function: any function returning the price of an option
        :type pricing_function: python function object

    """

    if t == 0:
        return POSINF if S == K else 0.0        

    return (pricing_function(flag, S + dS, K, t, r, sigma, b) - 2. * \
            pricing_function(flag, S, K, t, r, sigma, b) + \
            pricing_function(flag, S - dS, K, t, r, sigma, b)) / dS ** 2.



# -*- coding: utf-8 -*-
"""
    vollib.black
    ~~~~~~~~~~~~~~~~
    
    Copyright © 2015 
    
    A library for option pricing, implied volatility, and
    greek calculation.  vollib is based on lets_be_rational,
    a Python wrapper for LetsBeRational by Peter Jaeckel as 
    described below.
    
    :copyright: © 2015 Iota Technologies Pte Ltd    
    :license: MIT, see LICENSE for more details.
    
    About LetsBeRational:
    ~~~~~~~~~~~~~~~~~~~~~~~
    
    The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

    ::   

      ======================================================================================
      Copyright © 2013-2014 Peter Jäckel.
    
      Permission to use, copy, modify, and distribute this software is freely granted,
      provided that this notice is preserved.
   
      WARRANTY DISCLAIMER
      The Software is provided "as is" without warranty of any kind, either express or implied,
      including without limitation any implied warranties of condition, uninterrupted use,
      merchantability, fitness for a particular purpose, or non-infringement.
      ======================================================================================
    
"""
# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports
from math import e
from math import log, sqrt

# Related third party imports
import lets_be_rational
import numpy
from numpy import log, sqrt, exp


# Local application/library specific imports
from vollib.helper import binary_flag
from vollib.helper import pdf
from lets_be_rational import norm_cdf as cnd

# -----------------------------------------------------------------------------
# FUNCTIONS - INTERNAL, FOR COMPARISON

'''
From John C. Hull, "Options, Futures and Other Derivatives," 
7th edition, Chapter 16.8, page 342
'''

def d1(F, K, t, r, sigma): # keep r argument for consistency
    
    """Calculate the d1 component of the Black PDE.

    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float  
        
        
    Doctest using Hull, page 343, example 16.6
    
    >>> F = 20
    >>> K = 20
    >>> r = .09
    >>> t = 4/12.0
    >>> sigma = 0.25
    
    >>> calculated_value = d1(F, K, t, r, sigma) 
    >>> #0.0721687836487
    >>> text_book_value = 0.07216
    >>> abs(calculated_value - text_book_value) < .00001
    True
    """

    sigma_squared = sigma*sigma
    numerator = log(F/float(K)) + sigma_squared*t/2.0
    denominator = sigma*sqrt(t)

    return numerator/denominator

def d2(F, K, t, r, sigma): # keep r argument for consistency
    
    """Calculate the d2 component of the Black PDE.

    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float    
    
    Hull, page 343, example 16.6

    >>> F = 20
    >>> K = 20
    >>> r = .09
    >>> t = 4/12.0
    >>> sigma = 0.25

    >>> calculated_value = d2(F, K, t, r, sigma) 
    >>> #-0.0721687836487
    >>> text_book_value = -0.07216
    >>> abs(calculated_value - text_book_value) < .00001
    True
    """
    
    return d1(F, K, t, r, sigma) - sigma*sqrt(t)


def black_call(F, K, t, r, sigma):  #Equation 16.9
    

    """Calculate the price of a call using Black.  (Python
    implementation for reference.)

    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float    
    
    
    
    Hull, page 343, example 16.7
    
    >>> F = 620
    >>> K = 600
    >>> r = .05
    >>> sigma = .2
    >>> t = 0.5
    >>> calculated_value = black_call(F, K, t, r, sigma)
    >>> #44.1868533121
    >>> text_book_value = 44.19
    >>> abs(calculated_value - text_book_value) < .01
    True
    """
    
    e_to_the_minus_rt = numpy.exp(-r*t)
    N_d1 = cnd(d1(F, K, t, r, sigma))
    N_d2 = cnd(d2(F, K, t, r, sigma))
    
    return e_to_the_minus_rt * (F * N_d1 - K * N_d2) 

def black_put(F, K, t, r, sigma):  #Equation 16.10


    """Calculate the price of a put using Black.  (Python
    implementation for reference.)

    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float        
    
    
    Hull, page 338, example 16.6

    >>> F = 20
    >>> K = 20
    >>> r = .09
    >>> sigma = .25
    >>> t = 4/12.0
    >>> calculated_value = black_put(F, K, t, r, sigma)
    >>> # 1.11664145656
    >>> text_book_value = 1.12
    >>> abs(calculated_value - text_book_value) < .01
    True
    """
    e_to_the_minus_rt = numpy.exp(-r*t)
    N_of_minus_d1 = cnd(-d1(F, K, t, r, sigma))
    N_of_minus_d2 = cnd(-d2(F, K, t, r, sigma))

    return e_to_the_minus_rt * ( -F * N_of_minus_d1 + K * N_of_minus_d2)


# -----------------------------------------------------------------------------
# FUNCTIONS - FROM LETS_BE_RATIONAL

def undiscounted_black(F, K, sigma, t, flag):

    """Calculate the **undiscounted** Black option price.

    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float   

    >>> F = 100
    >>> K = 100
    >>> sigma = .2
    >>> flag = 'c'
    >>> t = .5
    >>> undiscounted_black(F, K, sigma, t, flag)
    5.637197779701664
    
    """

    q = binary_flag[flag]
    F = float(F)
    K = float(K)
    sigma = float(sigma)
    t = float(t)
    
    return lets_be_rational.black(F, K, sigma, t, q)


def black(flag, F, K, t, r, sigma):
    
    """Calculate the (discounted) Black option price.

    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float  
        
    >>> F = 100
    >>> K = 100
    >>> sigma = .2
    >>> flag = 'c'
    >>> r = .02
    >>> t = .5
    >>> black(flag, F, K, t, r, sigma)
    5.5811067246048118

    """
    
    discount_factor = e_to_the_minus_rt = numpy.exp(-r*t)        
    return undiscounted_black(F, K, sigma, t, flag) * discount_factor
    


def normalised_black(x, s, flag):

    """Calculate the normalised Black value,
    a time invariant transformation
    of the Black pricing formula.

    :param x: ln(F/K) where K is the strike price, and F is the futures price
    :type x: float
    :param s: volatility times the square root of time to expiration
    :type s: float
    :param flag: 'p' or 'c' for put or call 
    :type flag: str   
    
    >>> F = 100.
    >>> K = 95.
    >>> x = log(F/K)
    >>> t = 0.5
    >>> v = 0.3
    >>> s = v * sqrt(t)    
    
    >>> normalised_black(x,s,'p')
    0.061296663817558904
    
    >>> normalised_black(x,s,'c')
    0.11259558142181655
    
    """

    q = binary_flag[flag]
    
    return lets_be_rational.normalised_black(x, s, q)


def test_python_vs_c_values():
    
    #comment
    
    """
    >>> F = 100
    >>> K = 90
    >>> sigma = .2
    >>> flag = 'c'
    >>> r = .02
    >>> t = .5
    
    >>> epsilon = .000000001
    
    >>> v1 = black('c', F, K, t, r, sigma)
    >>> v2 = black_call(F, K, t, r, sigma)
    
    >>> print abs(v1-v2) < epsilon
    True
    
    
    >>> v1 = black('p', F, K, t, r, sigma)
    >>> v2 = black_put(F, K, t, r, sigma)
    >>> print abs(v1-v2) < epsilon
    True
    """
    pass



# -----------------------------------------------------------------------------
# MAIN
if __name__=='__main__':
    import doctest
    if not doctest.testmod().failed:
        print "Doctest passed"



# -*- coding: utf-8 -*-
"""
    vollib.black_scholes_merton.greeks.numerical
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    A library for option pricing, implied volatility, and
    greek calculation.  vollib is based on lets_be_rational,
    a Python wrapper for LetsBeRational by Peter Jaeckel as 
    described below.

    :copyright: © 2015 Iota Technologies Pte Ltd    
    :license: MIT, see LICENSE for more details.

    About LetsBeRational:
    ~~~~~~~~~~~~~~~~~~~~~~~

    The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

    ::

      ======================================================================================
      Copyright © 2013-2014 Peter Jäckel.

      Permission to use, copy, modify, and distribute this software is freely granted,
      provided that this notice is preserved.

      WARRANTY DISCLAIMER
      The Software is provided "as is" without warranty of any kind, either express or implied,
      including without limitation any implied warranties of condition, uninterrupted use,
      merchantability, fitness for a particular purpose, or non-infringement.
      ======================================================================================

"""

# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports
import numpy

# Related third party imports
from matplotlib import pyplot as plt

# Local application/library specific imports
from vollib.black_scholes_merton import black_scholes_merton

# numerical greeks
from vollib.helper.numerical_greeks import delta as numerical_delta
from vollib.helper.numerical_greeks import vega as numerical_vega
from vollib.helper.numerical_greeks import theta as numerical_theta
from vollib.helper.numerical_greeks import rho as numerical_rho
from vollib.helper.numerical_greeks import gamma as numerical_gamma

# analytical greeks
from vollib.black_scholes_merton.greeks.analytical import gamma as agamma
from vollib.black_scholes_merton.greeks.analytical import delta as adelta
from vollib.black_scholes_merton.greeks.analytical import vega as avega
from vollib.black_scholes_merton.greeks.analytical import rho as arho
from vollib.black_scholes_merton.greeks.analytical import theta as atheta

# -----------------------------------------------------------------------------
# FUNCTIONS - NUMERICAL GREEK CALCULATION

f = lambda flag, S, K, t, r, sigma, q: black_scholes_merton(flag, S, K, t, r, sigma, q)

def delta(flag, S, K, t, r, sigma, q):

    """Returns the Black-Scholes-Merton delta of an option.
    
    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float
    :param q: annualized continuous dividend yield
    :type q: float
    
    :returns:  float 
    """   


    return numerical_delta(flag, S, K, t, r, sigma, q, f)

def theta(flag, S, K, t, r, sigma, q):

    """Returns the Black-Scholes-Merton theta of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float
    :param q: annualized continuous dividend yield
    :type q: float

    :returns:  float 
    """   

    return numerical_theta(flag, S, K, t, r, sigma, q, f)

def vega(flag, S, K, t, r, sigma, q):

    """Returns the Black-Scholes-Merton vega of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float
    :param q: annualized continuous dividend yield
    :type q: float

    :returns:  float 
    """   
    return numerical_vega(flag, S, K, t, r, sigma, q, f)

def rho(flag, S, K, t, r, sigma, q):

    """Returns the Black-Scholes-Merton rho of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float
    :param q: annualized continuous dividend yield
    :type q: float

    :returns:  float 
    """   
    return numerical_rho(flag, S, K, t, r, sigma, q, f)

def gamma(flag, S, K, t, r, sigma, q):

    """Returns the Black-Scholes-Merton gamma of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float
    :param q: annualized continuous dividend yield
    :type q: float

    :returns:  float 
    """   
    return numerical_gamma(flag, S, K, t, r, sigma, q, f)


def plot_numerical_vs_analytical(greek, flag):
    
    """Displays a plot comparing analytically and numerically
    computed values of a given greek and option type.
    
    
    :param flag: 'd','g','t','r', or 'v'
    :type flag: str
    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    
    
        """


    f_analytical, f_numerical = {
        'd':(adelta,delta),
        'g':(agamma,gamma),
        't':(atheta,theta),
        'v':(avega,vega),
        'r':(arho, rho),
        }[greek]


    S_vals = numpy.linspace(10,250,2000)
    K=100
    t =.5
    q =.05
    sigma = 0.2
    r = .01


    price, analytical, numerical = [],[],[]


    for S in S_vals:
        analytical.append(f_analytical(flag, S, K, t, r, sigma, q))
        numerical.append(f_numerical(flag, S, K, t, r, sigma, q))
        price.append(black_scholes_merton(flag, S, K, t, r, sigma, q))
    plt.figure(1)
    plt.subplot(211)
    plt.plot(S_vals,analytical,label = 'Analytical')
    plt.plot(S_vals,numerical, label = 'Numerical')
    min_a, max_a = min(analytical), max(analytical)
    h = max_a-min_a
    plt.ylim(min_a - h*.05, max_a + h*.05)
    plt.grid()
    plt.legend(loc = 'best')

    plt.subplot(212)
    plt.plot(S_vals,price, label = 'Price')
    plt.ylim(min(min(price),-10), max(max(price),10))
    plt.grid()
    plt.legend(loc = 'best')


    plt.show()


#plot_numerical_vs_analytical('v','c')




# -----------------------------------------------------------------------------
# MAIN
if __name__=='__main__':  
    print 'running doctests'
    import doctest
    if not doctest.testmod().failed:
        print "Doctest passed"



# -*- coding: utf-8 -*-
"""
    vollib.black.greeks.numerical
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    A library for option pricing, implied volatility, and
    greek calculation.  vollib is based on lets_be_rational,
    a Python wrapper for LetsBeRational by Peter Jaeckel as 
    described below.

    :copyright: © 2015 Iota Technologies Pte Ltd    
    :license: MIT, see LICENSE for more details.

    About LetsBeRational:
    ~~~~~~~~~~~~~~~~~~~~~~~

    The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

    ::

      ======================================================================================
      Copyright © 2013-2014 Peter Jäckel.

      Permission to use, copy, modify, and distribute this software is freely granted,
      provided that this notice is preserved.

      WARRANTY DISCLAIMER
      The Software is provided "as is" without warranty of any kind, either express or implied,
      including without limitation any implied warranties of condition, uninterrupted use,
      merchantability, fitness for a particular purpose, or non-infringement.
      ======================================================================================

"""



import numpy
from matplotlib import pyplot as plt
from vollib.black import black
from vollib.helper.numerical_greeks import delta as numerical_delta
from vollib.helper.numerical_greeks import vega as numerical_vega
from vollib.helper.numerical_greeks import theta as numerical_theta
from vollib.helper.numerical_greeks import rho as numerical_rho
from vollib.helper.numerical_greeks import gamma as numerical_gamma

from vollib.black.greeks.analytical import gamma as agamma
from vollib.black.greeks.analytical import delta as adelta
from vollib.black.greeks.analytical import vega as avega
from vollib.black.greeks.analytical import rho as arho
from vollib.black.greeks.analytical import theta as atheta

f = lambda flag, F, K, t, r, sigma, b: black(flag, F, K, t, r, sigma)

def delta(flag, F, K, t, r, sigma):
    
    """Returns the Black delta of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float

    :returns:  float 
    """    
    
    
    b = 0
    
    return numerical_delta(flag, F, K, t, r, sigma, b, f)

def theta(flag, F, K, t, r, sigma):

    """Returns the Black theta of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float

    :returns:  float 
    """   


    b = 0

    return numerical_theta(flag, F, K, t, r, sigma, b, f)

def vega(flag, F, K, t, r, sigma):


    """Returns the Black vega of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float

    :returns:  float 
    """   
    
    b = 0

    return numerical_vega(flag, F, K, t, r, sigma, b, f)

def rho(flag, F, K, t, r, sigma):

    """Returns the Black rho of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float

    :returns:  float 
    """   

    b = 0

    return numerical_rho(flag, F, K, t, r, sigma, b, f)

def gamma(flag, F, K, t, r, sigma):

    """Returns the Black gamma of an option.

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float

    :returns:  float 
    """   

    b = 0

    return numerical_gamma(flag, F, K, t, r, sigma, b, f)



def test():
    
    '''Tests by comparing the analytical and numerical greek values.
    
    >>> S =  49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'

    >>> epsilon = .0001

    >>> v1 = delta(flag, S, K, t, r, sigma)
    >>> v2 = adelta(flag, S, K, t, r, sigma)
    >>> abs(v1-v2)<epsilon
    True

    >>> v1 = gamma(flag, S, K, t, r, sigma)
    >>> v2 = agamma(flag, S, K, t, r, sigma)
    >>> abs(v1-v2)<epsilon
    True

    >>> v1 = rho(flag, S, K, t, r, sigma)
    >>> v2 = arho(flag, S, K, t, r, sigma)
    >>> abs(v1-v2)<epsilon
    True

    >>> v1 = vega(flag, S, K, t, r, sigma)
    >>> v2 = avega(flag, S, K, t, r, sigma)
    >>> abs(v1-v2)<epsilon
    True

    >>> v1 = theta(flag, S, K, t, r, sigma)
    >>> v2 = atheta(flag, S, K, t, r, sigma)
    >>> abs(v1-v2)<epsilon
    True
    '''
    pass



def plot_numerical_vs_analytical(greek, flag):

    f_analytical, f_numerical = {
    'd':(adelta,delta),
    'g':(agamma,gamma),
    't':(atheta,theta),
    'v':(avega,vega),
    'r':(arho, rho),
    }[greek]


    S_vals = numpy.linspace(10,250,2000)
    K = 100
    t = 1.0
    r = .1
    sigma = 0.3


    price, analytical, numerical = [],[],[]
    
    
    for S in S_vals:
        analytical.append(f_analytical(flag, S, K, t, r, sigma))
        numerical.append(f_numerical(flag, S, K, t, r, sigma))
        price.append(black(flag, S, K, t, r, sigma))
    plt.figure(1)
    plt.subplot(211)
    plt.plot(S_vals,analytical,label = 'Analytical')
    plt.plot(S_vals,numerical, label = 'Numerical')
    min_a, max_a = min(analytical), max(analytical)
    h = max_a-min_a
    plt.ylim(min_a - h*.05, max_a + h*.05)
    plt.grid()
    plt.legend(loc = 'best')
    
    plt.subplot(212)
    plt.plot(S_vals,price, label = 'Price')
    plt.ylim(min(min(price),-10), max(max(price),10))
    plt.grid()
    plt.legend(loc = 'best')


    plt.show()
    


# -----------------------------------------------------------------------------
# MAIN
if __name__=='__main__':  
    import doctest
    if not doctest.testmod().failed:
        print "Doctest passed"



from collections import OrderedDict
import simplejson as json
import pandas



def almost_equal(a,b,epsilon = 1.0e-7):
    return abs(a-b)< epsilon


class TestDataIterator(object):

    """
    >>> data_iterator = TestDataIterator()
    >>> print data_iterator.has_next()
    True
    >>> r = data_iterator.next_row()
    >>> print r['S']
    100.0
    """
    
    def __init__(self):
        self.data = json.load(open('test_data.json','rb'))
        columns = self.data['columns']
        grid_data = OrderedDict()

        for header in columns:
            grid_data[header]=[]

        for col in self.data['data']:
            for i in range(len(columns)):
                grid_data[columns[i]].append(col[i])

        self.df = pandas.DataFrame(grid_data)
        self.row_id = 0
        self.row_count = self.df.S.count()

    def next_row(self):
        if self.has_next():
            row = self.df.ix[self.row_id].to_dict()
            self.row_id +=1
            return row

    def has_next(self):
        return self.row_id < self.row_count
    
    
# -----------------------------------------------------------------------------
# MAIN
if __name__=='__main__':  
    import doctest
    if not doctest.testmod().failed:
        print "Doctest passed"


from matplotlib import pyplot as plt
import numpy
from numpy import log, sqrt, exp
from scipy.stats import norm
cnd = norm.cdf
pdf = norm.pdf


from vollib.black_scholes import black_scholes as swig
from cython_black_scholes import BlackScholes as cython_bs
from vollib.black_scholes.greeks import gamma, delta, theta, rho, vega, gamma_analytical


vollib_bs = lambda flag, S, K, t, r, v: swig(S, K, v, t, r, flag)


def d1(S,K,t,r,v):

    v_squared = v*v
    numerator = log(S/float(K)) + (r + v_squared/2.)*t
    denominator = v*sqrt(t)

    return numerator/denominator

def d2(S,K,t,r,v):

    return d1(S, K, t, r, v) - v*sqrt(t)

def black_scholes(flag, S, K, t, r, v):

    cnd_d1 = cnd(d1(S, K, t, r, v))
    cnd_d2 = cnd(d2(S, K, t, r, v))
    e_to_the_minus_rt = numpy.exp(-r*t)

    if flag == 'c':

        return S * cnd_d1 - K * e_to_the_minus_rt * cnd_d2

    else:

        return - S * cnd_d1 + K * e_to_the_minus_rt * cnd_d2


def analytical_delta(flag, S, K, t, r, v):

    d_1 = d1(S, K, t, r, v)

    if flag == 'p':
        return cnd(d_1) -1.0
    else:
        return cnd(d_1)


def analytical_gamma(flag, S, K, t, r, v):

    d_1 = d1(S, K, t, r, v)
    v_squared = v**2
    return pdf(d_1)/(S*v*sqrt(t))


def analytical_vega(flag, S, K, t, r, v):

    '''
    The text book analytical formula does not multiply by .01,
    but in practice vega is defined as the change in price
    for each 1 percent change in IV, hence we multiply by 0.01.
    '''

    d_1 = d1(S, K, t, r, v)
    return S * pdf(d_1) * sqrt(t) * 0.01


def analytical_rho(flag, S, K, t, r, v):

    d_2 = d2(S, K, t, r, v)

    e_to_the_minus_rt = numpy.exp(-r*t)
    if flag == 'c':
        return t*K*e_to_the_minus_rt * cnd(d_2) * .01
    else:
        -t*K*e_to_the_minus_rt * cnd(-d_2) * .01


def analytical_theta(flag, S, K, t, r, v):

    '''
    The text book analytical formula does not divide by 365,
    but in practice theta is defined as the change in price
    for each day change in t, hence we divide by 365.
    '''

    e_to_the_minus_rt = numpy.exp(-r*t)
    two_sqrt_t = 2 * sqrt(t)

    d_1 = d1(S, K, t, r, v)
    d_2 = d2(S, K, t, r, v)
    pdf_d1 = pdf(d_1)
    cnd_d2 = cnd(d_2)

    first_term = (-S * pdf_d1 * v) / two_sqrt_t
    second_term = r * K * e_to_the_minus_rt * cnd_d2

    return (first_term - second_term)/365.0



def plot_swig_delta_vs_cython_call_delta():

    S_vals = numpy.linspace(10,190,2000)
    K = 100
    t = 1.0
    r = .1
    v = 0.3
    flag = 'c'

    call_delta_haug, call_delta_jaeckel, analytical = [],[],[]

    for S in S_vals:
        call_delta_haug.append(delta(flag,S,K,t,r,v,cython_bs))
        call_delta_jaeckel.append(delta(flag,S,K,t,r,v,vollib_bs))
        analytical.append(analytical_delta(flag, S, K, t, r, v))
    plt.plot(S_vals,call_delta_haug,label = 'Haug')
    plt.plot(S_vals,call_delta_jaeckel, label = 'Jaeckel')
    plt.plot(S_vals,analytical,label = 'analytical')
    plt.grid()
    plt.legend(loc = 'best')
    plt.show()

    diff_jaeckel = numpy.array(analytical) - numpy.array(call_delta_jaeckel)
    plt.clf()
    plt.plot(S_vals,diff_jaeckel)
    plt.grid()
    plt.show()



def plot_swig_delta_vs_cython_put_delta():


    S_vals = numpy.linspace(10,190,2000)
    K = 100
    t = 0.25
    r = .1
    v = 0.3
    flag = 'p'

    call_delta_haug, call_delta_jaeckel = [],[]

    for S in S_vals:
        call_delta_haug.append(delta(flag,S,K,t,r,v,cython_bs))
        call_delta_jaeckel.append(delta(flag,S,K,t,r,v,vollib_bs))
    plt.plot(S_vals,call_delta_haug,label = 'Haug')
    plt.plot(S_vals,call_delta_jaeckel, label = 'Jaeckel')
    plt.grid()

    plt.legend(loc = 'best')
    plt.show()

    diff = numpy.array(call_delta_haug) - numpy.array(call_delta_jaeckel)
    plt.clf()
    plt.plot(S_vals,diff)
    plt.grid()
    plt.show()

def plot_swig_delta_vs_cython_call_gamma():

    S_vals = numpy.linspace(10,190,2000)
    K = 100
    t = 1
    r = .1
    v = 0.3
    flag = 'c'

    call_gamma_haug, call_gamma_jaeckel, analytical = [],[],[]

    for S in S_vals:
        call_gamma_haug.append(gamma(flag,S,K,t,r,v,cython_bs))
        call_gamma_jaeckel.append(gamma(flag,S,K,t,r,v,vollib_bs))
        analytical.append(analytical_gamma(flag, S, K, t, r, v))
    plt.plot(S_vals,call_gamma_haug,label = 'Haug')
    plt.plot(S_vals,call_gamma_jaeckel, label = 'Jaeckel')
    plt.plot(S_vals,analytical,label = 'analytical', linewidth = 5)
    plt.grid()
    plt.legend(loc = 'best')
    plt.show()

    diff = numpy.array(analytical) - numpy.array(call_gamma_jaeckel)
    plt.clf()
    plt.plot(S_vals,diff)
    plt.grid()
    plt.show()

def plot_swig_delta_vs_cython_vega():

    S_vals = numpy.linspace(10,190,2000)
    K = 100
    t = 1
    r = .1
    v = 0.3
    flag = 'c'

    haug, jaeckel, analytical = [],[],[]

    for S in S_vals:
        haug.append(vega(flag,S,K,t,r,v,cython_bs))
        jaeckel.append(vega(flag,S,K,t,r,v,vollib_bs))
        analytical.append(analytical_vega(flag, S, K, t, r, v))
    plt.plot(S_vals,haug,label = 'Haug')
    plt.plot(S_vals,jaeckel, label = 'Jaeckel')
    plt.plot(S_vals,analytical,label = 'analytical')
    plt.grid()
    plt.legend(loc = 'best')
    plt.show()

    diff = numpy.array(analytical) - numpy.array(jaeckel)
    plt.clf()
    plt.plot(S_vals,diff)
    plt.grid()
    plt.show()

def plot_swig_delta_vs_cython_theta():

    S_vals = numpy.linspace(10,190,2000)
    K = 100
    t = 1
    r = .1
    v = 0.3
    flag = 'c'

    haug, jaeckel, analytical, python = [],[],[],[]

    for S in S_vals:
        haug.append(theta(flag,S,K,t,r,v,cython_bs))
        jaeckel.append(theta(flag,S,K,t,r,v,vollib_bs))
        python.append(theta(flag, S, K, t, r, v, black_scholes))
        analytical.append(analytical_theta(flag, S, K, t, r, v))
    plt.plot(S_vals,haug,label = 'Haug')
    plt.plot(S_vals,jaeckel, label = 'Jaeckel')
    plt.plot(S_vals,analytical,label = 'analytical')
    plt.plot(S_vals,python,label = 'python')

    plt.grid()
    plt.legend(loc = 'best')
    plt.show()

    diff = numpy.array(haug) - numpy.array(jaeckel)
    plt.clf()
    plt.plot(S_vals,diff)
    plt.grid()
    plt.show()

def plot_swig_vs_cython_rho():

    S_vals = numpy.linspace(10,190,2000)
    K = 100
    t = 1
    r = .1
    v = 0.3
    flag = 'c'

    haug, jaeckel, analytical, python = [],[],[],[]

    for S in S_vals:
        haug.append(rho(flag,S,K,t,r,v,cython_bs))
        jaeckel.append(rho(flag,S,K,t,r,v,vollib_bs))
        python.append(rho(flag, S, K, t, r, v, black_scholes))
        analytical.append(analytical_rho(flag, S, K, t, r, v))
    plt.plot(S_vals,haug,label = 'Haug')
    plt.plot(S_vals,jaeckel, label = 'Jaeckel')
    plt.plot(S_vals,analytical,label = 'analytical')
    plt.plot(S_vals,python,label = 'python')

    plt.grid()
    plt.legend(loc = 'best')
    plt.show()

    diff = numpy.array(haug) - numpy.array(analytical)
    plt.clf()
    plt.plot(S_vals,diff)
    plt.grid()
    plt.show()


plot_swig_delta_vs_cython_theta()



# -*- coding: utf-8 -*-
"""
    vollib.black_scholes.greeks.numerical
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    A library for option pricing, implied volatility, and
    greek calculation.  vollib is based on lets_be_rational,
    a Python wrapper for LetsBeRational by Peter Jaeckel as 
    described below.

    :copyright: © 2015 Iota Technologies Pte Ltd    
    :license: MIT, see LICENSE for more details.

    About LetsBeRational:
    ~~~~~~~~~~~~~~~~~~~~~~~

    The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

    ::

      ======================================================================================
      Copyright © 2013-2014 Peter Jäckel.

      Permission to use, copy, modify, and distribute this software is freely granted,
      provided that this notice is preserved.

      WARRANTY DISCLAIMER
      The Software is provided "as is" without warranty of any kind, either express or implied,
      including without limitation any implied warranties of condition, uninterrupted use,
      merchantability, fitness for a particular purpose, or non-infringement.
      ======================================================================================

"""

# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports
import numpy

# Related third party imports
from matplotlib import pyplot as plt

# Local application/library specific imports
from vollib.black_scholes import black_scholes

# numerical greeks
from vollib.helper.numerical_greeks import delta as numerical_delta
from vollib.helper.numerical_greeks import vega as numerical_vega
from vollib.helper.numerical_greeks import theta as numerical_theta
from vollib.helper.numerical_greeks import rho as numerical_rho
from vollib.helper.numerical_greeks import gamma as numerical_gamma

# analytical greeks
from vollib.black_scholes.greeks.analytical import gamma as agamma
from vollib.black_scholes.greeks.analytical import delta as adelta
from vollib.black_scholes.greeks.analytical import vega as avega
from vollib.black_scholes.greeks.analytical import rho as arho
from vollib.black_scholes.greeks.analytical import theta as atheta

# -----------------------------------------------------------------------------
# FUNCTIONS - NUMERICAL GREEK CALCULATION


f = lambda flag, S, K, t, r, sigma, b: black_scholes(flag, S, K, t, r, sigma)


def delta(flag, S, K, t, r, sigma):
    
    """Return Black-Scholes delta of an option.
    
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param flag: 'c' or 'p' for call or put.
    :type flag: str      
    
    """
    
    b = r

    return numerical_delta(flag, S, K, t, r, sigma, b, f)

def theta(flag, S, K, t, r, sigma):
    
    """Return Black-Scholes theta of an option.

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param flag: 'c' or 'p' for call or put.
    :type flag: str      

    """    
    
    b = r

    return numerical_theta(flag, S, K, t, r, sigma, b, f)

def vega(flag, S, K, t, r, sigma):

    """Return Black-Scholes vega of an option.

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param flag: 'c' or 'p' for call or put.
    :type flag: str      

    """

    b = r

    return numerical_vega(flag, S, K, t, r, sigma, b, f)

def rho(flag, S, K, t, r, sigma):
    
    """Return Black-Scholes rho of an option.

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param flag: 'c' or 'p' for call or put.
    :type flag: str      

    """    
    
    b = r

    return numerical_rho(flag, S, K, t, r, sigma, b, f)

def gamma(flag, S, K, t, r, sigma):

    """Return Black-Scholes gamma of an option.

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param flag: 'c' or 'p' for call or put.
    :type flag: str      

    """

    b = r

    return numerical_gamma(flag, S, K, t, r, sigma, b, f)


def test():
    """Test by comparing analytical and numerical values.
    
    >>> S =  49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    
    >>> epsilon = .0001
    
    >>> v1 = delta(flag, S, K, t, r, sigma)
    >>> v2 = adelta(flag, S, K, t, r, sigma)
    >>> abs(v1-v2)<epsilon
    True
    
    >>> v1 = gamma(flag, S, K, t, r, sigma)
    >>> v2 = agamma(flag, S, K, t, r, sigma)
    >>> abs(v1-v2)<epsilon
    True
    
    >>> v1 = rho(flag, S, K, t, r, sigma)
    >>> v2 = arho(flag, S, K, t, r, sigma)
    >>> abs(v1-v2)<epsilon
    True
    
    >>> v1 = vega(flag, S, K, t, r, sigma)
    >>> v2 = avega(flag, S, K, t, r, sigma)
    >>> abs(v1-v2)<epsilon
    True
    
    >>> v1 = theta(flag, S, K, t, r, sigma)
    >>> v2 = atheta(flag, S, K, t, r, sigma)
    >>> abs(v1-v2)<epsilon
    True
    """
    pass





def hull_book_tests():
    """
    Example 17.1, page 355, Hull:

    >>> S = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> delta_calc = delta(flag, S, K, t, r, sigma)
    >>> # 0.521601633972
    >>> delta_text_book = 0.522
    >>> abs(delta_calc - delta_text_book) < .01
    True


    Example 17.2, page 359, Hull:

    >>> S = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> annual_theta_calc = theta(flag, S, K, t, r, sigma) * 365
    >>> # -4.30538996455
    >>> annual_theta_text_book = -4.31
    >>> abs(annual_theta_calc - annual_theta_text_book) < .01
    True
    
    
    Example 17.4, page 364, Hull:

    >>> S = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> gamma_calc = gamma(flag, S, K, t, r, sigma)
    >>> # 0.0655453772525
    >>> gamma_text_book = 0.066
    >>> abs(gamma_calc - gamma_text_book) < .001
    True
    
    
    Example 17.6, page 367, Hull:
    
    >>> S = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> vega_calc = vega(flag, S, K, t, r, sigma)
    >>> # 0.121052427542
    >>> vega_text_book = 0.121
    >>> abs(vega_calc - vega_text_book) < .01
    True

    Example 17.7, page 368, Hull:

    >>> S = 49
    >>> K = 50 
    >>> r = .05
    >>> t = 0.3846
    >>> sigma = 0.2
    >>> flag = 'c'
    >>> rho_calc = rho(flag, S, K, t, r, sigma)
    >>> # 0.089065740988
    >>> rho_text_book = 0.0891
    >>> abs(rho_calc - rho_text_book) < .0001
    True

    """






# -----------------------------------------------------------------------------
# MAIN
if __name__=='__main__':  
    print 'running doctests'
    import doctest
    if not doctest.testmod().failed:
        print "Doctest passed"



# -*- coding: utf-8 -*-
"""
    vollib.helper
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    A library for option pricing, implied volatility, and
    greek calculation.  vollib is based on lets_be_rational,
    a Python wrapper for LetsBeRational by Peter Jaeckel as 
    described below.

    :copyright: © 2015 Iota Technologies Pte Ltd    
    :license: MIT, see LICENSE for more details.

    About LetsBeRational:
    ~~~~~~~~~~~~~~~~~~~~~~~

    The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

    ::

      ======================================================================================
      Copyright © 2013-2014 Peter Jäckel.

      Permission to use, copy, modify, and distribute this software is freely granted,
      provided that this notice is preserved.

      WARRANTY DISCLAIMER
      The Software is provided "as is" without warranty of any kind, either express or implied,
      including without limitation any implied warranties of condition, uninterrupted use,
      merchantability, fitness for a particular purpose, or non-infringement.
      ======================================================================================

"""


# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports
import numpy
from numpy import log, sqrt, exp

# -----------------------------------------------------------------------------
# DATA

ONE_OVER_SQRT_TWO_PI = 0.3989422804014326779399460599343818684758586311649
CALL = 'c'
PUT = 'p'


binary_flag = {CALL:1,PUT:-1}
"""
::

  ========================================================

  Note:  In "Let's be Rational," Peter Jäckel uses θ as a flag
  to distinguish between puts and calls.
  +1 represents a call, -1 represents a put.

  See page 1, Introduction, first paragraph.
  ========================================================

>>> binary_flag['c']:
1
>>> binary_flag['c']:
-1


"""
# -----------------------------------------------------------------------------
# FUNCTIONS

pdf = lambda x: ONE_OVER_SQRT_TWO_PI * numpy.exp(-.5*x*x)
"""the probability density function

    :param x: a continuous random variable
    :type S: float

"""


def forward_price(S,t,r):
    

    """Calculate the forward price of an underlying asset.

    :param S: underlying asset price
    :type S: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
  

    >>> S = 95
    >>> t = .5
    >>> r = .02
    >>> forward_price(S,t,r)
    95.95476587299596
    """
    return S/numpy.exp(-r*t)



# -----------------------------------------------------------------------------
# MAIN
if __name__=='__main__':
    import doctest
    if not doctest.testmod().failed:
        print "Doctest passed"



# -*- coding: utf-8 -*-
"""
    vollib.black_scholes.implied_volatility
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    A library for option pricing, implied volatility, and
    greek calculation.  vollib is based on lets_be_rational,
    a Python wrapper for LetsBeRational by Peter Jaeckel as 
    described below.

    :copyright: © 2015 Iota Technologies Pte Ltd    
    :license: MIT, see LICENSE for more details.

    About LetsBeRational:
    ~~~~~~~~~~~~~~~~~~~~~~~

    The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

    :: 

      ======================================================================================
      Copyright © 2013-2014 Peter Jäckel.

      Permission to use, copy, modify, and distribute this software is freely granted,
      provided that this notice is preserved.

      WARRANTY DISCLAIMER
      The Software is provided "as is" without warranty of any kind, either express or implied,
      including without limitation any implied warranties of condition, uninterrupted use,
      merchantability, fitness for a particular purpose, or non-infringement.
      ======================================================================================

"""



# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports

# Related third party imports
import lets_be_rational
from scipy.optimize import brentq
import numpy

# Local application/library specific imports
from vollib.helper import forward_price
from vollib.black_scholes import black_scholes
from vollib.helper import binary_flag


e = numpy.e

# -----------------------------------------------------------------------------
# FUNCTIONS, FOR REFERENCE AND TESTING


def implied_volatility_brent(price, S, K, t, r, flag):

    """Calculate the Black-Scholes implied volatility
    using the Brent method (for reference).

    :param price: the Black-Scholes option price
    :type price: float
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param flag: 'c' or 'p' for call or put.
    :type flag: str  
    
    
    >>> S = 100
    >>> K = 100
    >>> sigma = .2
    >>> r = .01
    >>> flag = 'c'
    >>> t = .5

    >>> price = black_scholes(flag, S, K, t, r, sigma)
    >>> iv = implied_volatility_brent(price, S, K, t, r, flag)

    >>> print price, iv
    5.87602423383 0.2
    """  

    def function_to_minimize(sigma):
        return price - black_scholes(flag, S, K, t, r, sigma)

    return brentq(function_to_minimize,0,10)  


def implied_volatility_limited_iterations(price, S, K, t, r, flag, N):

    """Calculate the Black-Scholes implied volatility with limited iterations.

    :param price: the Black-Scholes option price
    :type price: float
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param flag: 'c' or 'p' for call or put.
    :type flag: str  
    :param N: the maximum number of iterations to perform
    :type N: int

    >>> S = 100
    >>> K = 100
    >>> sigma = .232323232
    >>> r = .01
    >>> flag = 'c'
    >>> t = .5

    >>> price = black_scholes(flag, S, K, t, r, sigma)
    >>> iv = implied_volatility_limited_iterations(price, S, K, t, r, flag,1)

    >>> print price, iv    
    6.78242400926 0.232323232
    """  

    adjusted_price = price / e**(-r*t)

    return lets_be_rational.implied_volatility_from_a_transformed_rational_guess_with_limited_iterations(
        adjusted_price, 
        forward_price(S, t, r), 
        K, 
        t, 
        binary_flag[flag],
        N
    )


# -----------------------------------------------------------------------------
# FUNCTIONS


def implied_volatility(price, S, K, t, r, flag):


    """Calculate the Black-Scholes implied volatility.

    :param price: the Black-Scholes option price
    :type price: float
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param flag: 'c' or 'p' for call or put.
    :type flag: str 
    
    >>> S = 100
    >>> K = 100
    >>> sigma = .2
    >>> r = .01
    >>> flag = 'c'
    >>> t = .5

    >>> price = black_scholes(flag, S, K, t, r, sigma)
    >>> iv = implied_volatility(price, S, K, t, r, flag)

    >>> print price, iv
    5.87602423383 0.2
    """  

    adjusted_price = price / e**(-r*t)

    return lets_be_rational.implied_volatility_from_a_transformed_rational_guess(
        adjusted_price, 
        forward_price(S, t, r), 
        K, 
        t, 
        binary_flag[flag]
    )
    
# -----------------------------------------------------------------------------
# MAIN
if __name__=='__main__':
    import doctest
    if not doctest.testmod().failed:
        print "Doctest passed"


import unittest

from vollib.tests.test_utils import TestDataIterator, almost_equal
from vollib.black_scholes import black_scholes
from vollib.black_scholes.implied_volatility import implied_volatility
from vollib.black_scholes.greeks import analytical
from vollib.black_scholes.greeks import numerical


class TestBlackScholesAgainstBenchmarkValues(unittest.TestCase):
    
    def setUp(self):
        self.tdi = TestDataIterator()

    def test_prices(self):

        while self.tdi.has_next():
            row = self.tdi.next_row()
            S,K,t,r,sigma = row['S'],row['K'],row['t'],row['R'],row['v']
            self.assertTrue(
                almost_equal(
                    black_scholes('c', S, K, t, r, sigma), row['bs_call'], epsilon=.000001
                )
            )
            self.assertTrue(
                almost_equal(
                    black_scholes('p', S, K, t, r, sigma), row['bs_put'], epsilon=.000001
                )
            )
            
    
    def test_analytical_delta(self):
        
        while self.tdi.has_next():
            row = self.tdi.next_row()
            S,K,t,r,sigma = row['S'],row['K'],row['t'],row['R'],row['v']
            self.assertTrue(
                almost_equal(
                    analytical.delta('c', S, K, t, r, sigma), row['CD'], epsilon=.000001
                )
            )
            self.assertTrue(
                almost_equal(
                    analytical.delta('p', S, K, t, r, sigma), row['PD'], epsilon=.000001
                )
            )
        
    def _test_analytical_theta(self):

        while self.tdi.has_next():
            row = self.tdi.next_row()
            S,K,t,r,sigma = row['S'],row['K'],row['t'],row['R'],row['v']
            #self.assertTrue(
            #    almost_equal(
            print        analytical.theta('c', S, K, t, r, sigma), row['CT']#, epsilon=.001
            #    )
            #)
            #self.assertTrue(
            #    almost_equal(
            print         analytical.theta('p', S, K, t, r, sigma), row['PT'] #, epsilon=.000001
            #    )
            #)
            
            
    def test_analytical_gamma(self):

        while self.tdi.has_next():
            row = self.tdi.next_row()
            S,K,t,r,sigma = row['S'],row['K'],row['t'],row['R'],row['v']
            self.assertTrue(
                almost_equal(
                    analytical.gamma('c', S, K, t, r, sigma), row['CG'], epsilon=.000001
                )
            )
            self.assertTrue(
                almost_equal(
                     analytical.gamma('p', S, K, t, r, sigma), row['PG'], epsilon=.000001
                )
            )
            
    def test_analytical_vega(self):

        while self.tdi.has_next():
            row = self.tdi.next_row()
            S,K,t,r,sigma = row['S'],row['K'],row['t'],row['R'],row['v']
            self.assertTrue(
                almost_equal(
                    analytical.vega('c', S, K, t, r, sigma), row['CV']*.01, epsilon=.01
                )
            )
            self.assertTrue(
                almost_equal(
                    analytical.vega('p', S, K, t, r, sigma), row['PV']*.01, epsilon=.01
                )
            )
            
            
    def test_analytical_rho(self):

        while self.tdi.has_next():
            row = self.tdi.next_row()
            S,K,t,r,sigma = row['S'],row['K'],row['t'],row['R'],row['v']
            self.assertTrue(
                almost_equal(
                    analytical.rho('c', S, K, t, r, sigma), row['CR']*.01, epsilon=.000000001
                )
            )
            self.assertTrue(
                almost_equal(
                    analytical.rho('p', S, K, t, r, sigma), row['PR']*.01, epsilon=.000000001
                )
            )
    
    
    def test_implied_volatility(self):
        
        while self.tdi.has_next():
            
            row = self.tdi.next_row()
            S,K,t,r,sigma = row['S'],row['K'],row['t'],row['R'],row['v']
            C,P = black_scholes('c', S, K, t, r, sigma), black_scholes('p', S, K, t, r, sigma)
            try:
                iv = implied_volatility(C, S, K, t, r, 'c')
                self.assertTrue(almost_equal(sigma, iv, epsilon = .0001))
            except:
                print 'could not calculate iv for ', C, S, K, t, r, 'c' 

            iv = implied_volatility(P, S, K, t, r, 'p')
            self.assertTrue(almost_equal(sigma, iv, epsilon = .001) or (iv ==0.0))


    
if __name__ == '__main__':
    unittest.main()




# -*- coding: utf-8 -*-
"""
    vollib
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    A library for option pricing, implied volatility, and
    greek calculation.  vollib is based on lets_be_rational,
    a Python wrapper for LetsBeRational by Peter Jaeckel as 
    described below.

    :copyright: © 2015 Iota Technologies Pte Ltd    
    :license: MIT, see LICENSE for more details.

    About LetsBeRational:
    ~~~~~~~~~~~~~~~~~~~~~~~

    The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

    ::

      ========================================================================================
      Copyright © 2013-2014 Peter Jäckel.

      Permission to use, copy, modify, and distribute this software is freely granted,
      provided that this notice is preserved.

      WARRANTY DISCLAIMER
      The Software is provided "as is" without warranty of any kind, either express or implied,
      including without limitation any implied warranties of condition, uninterrupted use,
      merchantability, fitness for a particular purpose, or non-infringement.
      ========================================================================================

"""
# -----------------------------------------------------------------------------
# IMPORTS



# -*- coding: utf-8 -*-
"""
    vollib.black_scholes_merton.implied_volatility
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Copyright © 2015 Iota Technologies Pte Ltd

    A library for option pricing, implied volatility, and
    greek calculation.  vollib is based on lets_be_rational,
    a Python wrapper for LetsBeRational by Peter Jaeckel as 
    described below.

    :copyright: © 2015 Iota Technologies Pte Ltd    
    :license: MIT, see LICENSE for more details.

    About LetsBeRational:
    ~~~~~~~~~~~~~~~~~~~~~~~

    The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

    ::

      ======================================================================================
      Copyright © 2013-2014 Peter Jäckel.

      Permission to use, copy, modify, and distribute this software is freely granted,
      provided that this notice is preserved.

      WARRANTY DISCLAIMER
      The Software is provided "as is" without warranty of any kind, either express or implied,
      including without limitation any implied warranties of condition, uninterrupted use,
      merchantability, fitness for a particular purpose, or non-infringement.
      ======================================================================================

"""


# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports

# Related third party imports
from lets_be_rational import implied_volatility_from_a_transformed_rational_guess as iv
from scipy.optimize import brentq
import numpy

# Local application/library specific imports
from vollib.helper import forward_price
from vollib.black_scholes_merton import black_scholes_merton
from vollib.black_scholes_merton import python_black_scholes_merton
from vollib.helper import binary_flag

# -----------------------------------------------------------------------------
# FUNCTIONS, FOR REFERENCE AND TESTING


def implied_volatility_brent(price, S, K, t, r, q, flag):

    """Calculate the Black-Scholes-Merton implied volatility
    using the Brent method.  (For comparison)

    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param q: annualized continuous dividend rate
    :type q: float 
    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    
    >>> S = 100
    >>> K = 100
    >>> sigma = .2
    >>> r = .01
    >>> flag = 'c'
    >>> t = .5
    >>> q = .02
    

    >>> price = black_scholes_merton(flag, S, K, t, r, sigma, q)
    >>> iv = implied_volatility_brent(price, S, K, t, r, q, flag)

    >>> print abs(iv - 0.2) < .0000001
    True
    """  

    def function_to_minimize(sigma):
        return price - black_scholes_merton(flag, S, K, t, r, sigma, q)

    return brentq(function_to_minimize,0,10)  


# -----------------------------------------------------------------------------
# FUNCTIONS


def implied_volatility(price, S, K, t, r, q, flag):


    """Calculate the Black-Scholes-Merton implied volatility.


    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param sigma: annualized standard deviation, or volatility
    :type sigma: float
    :param t: time to expiration in years
    :type t: float
    :param r: risk-free interest rate
    :type r: float
    :param q: annualized continuous dividend rate
    :type q: float 
    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    
    
    >>> S = 100
    >>> K = 100
    >>> sigma = .2
    >>> r = .01
    >>> flag = 'c'
    >>> t = .5
    >>> q = .02
    
    >>> reference_price = python_black_scholes_merton(flag, S, K, t, r, sigma, q)
    >>> price = black_scholes_merton(flag, S, K, t, r, sigma, q)

    >>> abs(reference_price - price) < .00000001
    True
    >>> iv = implied_volatility(price, S, K, t, r, q, flag)
    >>> iv_brent = implied_volatility_brent(price, S, K, t, r, q, flag)

    >>> abs(iv - iv_brent) < .00000001
    True
    """  
    conversion_factor = numpy.exp(-r*t)
    adjusted_price = price / conversion_factor
    S = S * numpy.exp((r-q)*t)
    return iv(adjusted_price, S, K, t, binary_flag[flag])
    
# -----------------------------------------------------------------------------
# MAIN
if __name__=='__main__':
    import doctest
    if not doctest.testmod().failed:
        print "Doctest passed"



# -*- coding: utf-8 -*-
"""
    vollib.black.implied_volatility
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    A library for option pricing, implied volatility, and
    greek calculation.  vollib is based on lets_be_rational,
    a Python wrapper for LetsBeRational by Peter Jaeckel as 
    described below.

    :copyright: © 2015 Iota Technologies Pte Ltd    
    :license: MIT, see LICENSE for more details.

    About LetsBeRational:
    ~~~~~~~~~~~~~~~~~~~~~~~

    The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

    :: 

      ======================================================================================
      Copyright © 2013-2014 Peter Jäckel.

      Permission to use, copy, modify, and distribute this software is freely granted,
      provided that this notice is preserved.

      WARRANTY DISCLAIMER
      The Software is provided "as is" without warranty of any kind, either express or implied,
      including without limitation any implied warranties of condition, uninterrupted use,
      merchantability, fitness for a particular purpose, or non-infringement.
      ======================================================================================

"""

# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports
from math import e

# Related third party imports
import lets_be_rational
import numpy

# Local application/library specific imports
from vollib.black import black
from vollib.black import undiscounted_black
from vollib.black import normalised_black

# -----------------------------------------------------------------------------
# DATA

CALL = 'c'
PUT = 'p'
binary_flag = {CALL:1,PUT:-1}

# -----------------------------------------------------------------------------
# FUNCTIONS - IMPLIED VOLATILITY

def normalised_implied_volatility(beta, x, flag):

    """Calculate the normalised Black implied volatility,
    a time invariant transformation
    of Black implied volatility.

    Keyword arguments:
    
    :param x: ln(F/K) where K is the strike price, and F is the futures price
    :type x: float
    :param beta: the normalized Black price
    :type beta: float
    :param flag: 'p' or 'c' for put or call 
    :type flag: str       
    
    
    >>> beta_call = normalised_black(0.0, 0.2, 'c')
    >>> beta_put = normalised_black(0.1,0.23232323888,'p')
    >>> normalized_b76_iv_call = normalised_implied_volatility(beta_call, 0.0, 'c')
    >>> normalized_b76_iv_put = normalised_implied_volatility(beta_put, 0.1, 'p')
    >>> print beta_call, normalized_b76_iv_call
    0.0796556745541 0.2
    >>> print beta_put, normalized_b76_iv_put
    0.0509710222785 0.23232323888
    """    

    q = binary_flag[flag]
    return lets_be_rational.normalised_implied_volatility_from_a_transformed_rational_guess(
        beta, x, q)


def normalised_implied_volatility_limited_iterations(beta, x, flag, N):


    """Calculate the normalised Black implied volatility,
    with limited iterations.

    :param x: ln(F/K) where K is the strike price, and F is the futures price
    :type x: float
    :param beta: the normalized Black price
    :type beta: float
    :param flag: 'p' or 'c' for put or call 
    :type flag: str  

    >>> beta_call = normalised_black(0.0, 0.2, 'c')
    >>> beta_put = normalised_black(0.1,0.23232323888,'p')
    >>> normalized_b76_iv_call = normalised_implied_volatility_limited_iterations(beta_call, 0.0, 'c',1)
    >>> normalized_b76_iv_put = normalised_implied_volatility_limited_iterations(beta_put, 0.1, 'p',1)
    >>> print beta_call, normalized_b76_iv_call
    0.0796556745541 0.2
    >>> print beta_put, normalized_b76_iv_put
    0.0509710222785 0.23232323888
    """    

    q = binary_flag[flag]
    return lets_be_rational.normalised_implied_volatility_from_a_transformed_rational_guess_with_limited_iterations(
        beta, x, q, N)
        
def implied_volatility_of_undiscounted_option_price(undiscounted_option_price, F, K, t, flag):

    """Calculate the implied volatility of the undiscounted Black option price

    :param undiscounted_option_price: undiscounted Black price of a futures option
    :type undiscounted_option_price: float
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float  

    >>> F = 100
    >>> K = 100
    >>> sigma = .2
    >>> flag = 'c'
    >>> t = .5

    >>> undiscounted_call_price = undiscounted_black(F, K, sigma, t, flag)
    >>> iv = implied_volatility_of_undiscounted_option_price(
    ... undiscounted_call_price, F, K, t, flag)

    >>> print undiscounted_call_price, iv
    5.6371977797 0.2
    """

    return lets_be_rational.implied_volatility_from_a_transformed_rational_guess(
        undiscounted_option_price, 
        F,
        K, 
        t, 
        binary_flag[flag]
    )


def implied_volatility_of_discounted_option_price(discounted_option_price, F, K, r, t, flag):

    """Calculate the implied volatility of the Black option price

    :param discounted_option_price: discounted Black price of a futures option
    :type discounted_option_price: float
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param r: the risk-free interest rate
    :type r: float 
    :param t: time to expiration in years
    :type t: float
    :param flag: 'p' or 'c' for put or call
    :type flag: str


    >>> F = 100
    >>> K = 100
    >>> sigma = .2
    >>> flag = 'c'
    >>> t = .5
    >>> r = .02

    >>> discounted_call_price = black(flag, F, K, t, r, sigma)
    >>> iv = implied_volatility_of_discounted_option_price(
    ... discounted_call_price, F, K, r, t, flag)

    >>> print discounted_call_price, iv
    5.5811067246 0.2
    """
    
    discount_factor = numpy.exp(-r*t)
    undiscounted_option_price = discounted_option_price / discount_factor
    
    return lets_be_rational.implied_volatility_from_a_transformed_rational_guess(
        undiscounted_option_price, 
        F,
        K, 
        t, 
        binary_flag[flag]
    )




def implied_volatility_of_undiscounted_option_price_limited_iterations(
    undiscounted_option_price, F, K, t, flag, N):

    """Calculate implied volatility of the undiscounted Black 
    option price with limited iterations.

    :param undiscounted_option_price: undiscounted Black price of a futures option
    :type undiscounted_option_price: float
    :param F: underlying futures price
    :type F: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float 

    >>> F = 100
    >>> K = 100
    >>> sigma = .232323232
    >>> flag = 'c'
    >>> t = .5

    >>> price = undiscounted_black(F, K, sigma, t, flag)
    >>> iv = implied_volatility_of_undiscounted_option_price_limited_iterations(
    ... price, F, K, t, flag, 1)

    >>> print price, iv    
    6.54635543387 0.232323232
    """  

    return lets_be_rational.implied_volatility_from_a_transformed_rational_guess_with_limited_iterations(
        undiscounted_option_price, 
        F,
        K, 
        t, 
        binary_flag[flag],
        N
    )

# -----------------------------------------------------------------------------
# MAIN
if __name__=='__main__':
    import doctest
    if not doctest.testmod().failed:
        print "Doctest passed"



# -*- coding: utf-8 -*-
"""
    vollib.black_scholes_merton.greeks.analytical
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    A library for option pricing, implied volatility, and
    greek calculation.  vollib is based on lets_be_rational,
    a Python wrapper for LetsBeRational by Peter Jaeckel as 
    described below.

    :copyright: © 2015 Iota Technologies Pte Ltd    
    :license: MIT, see LICENSE for more details.

    About LetsBeRational:
    ~~~~~~~~~~~~~~~~~~~~~~~

    The source code of LetsBeRational resides at www.jaeckel.org/LetsBeRational.7z .

    ::

      ======================================================================================
      Copyright © 2013-2014 Peter Jäckel.

      Permission to use, copy, modify, and distribute this software is freely granted,
      provided that this notice is preserved.

      WARRANTY DISCLAIMER
      The Software is provided "as is" without warranty of any kind, either express or implied,
      including without limitation any implied warranties of condition, uninterrupted use,
      merchantability, fitness for a particular purpose, or non-infringement.
      ======================================================================================

"""


# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports


# Related third party imports
import numpy

# Local application/library specific imports
from lets_be_rational import norm_cdf as cnd
from vollib.helper import pdf
from vollib.black_scholes_merton import d1,d2

# -----------------------------------------------------------------------------
# FUNCTIONS - ANALYTICAL GREEKS
# -----------------------------------------------------------------------------
# IMPORTS

# Standard library imports


# Related third party imports
import numpy

# Local application/library specific imports
from lets_be_rational import norm_cdf as cnd
from vollib.helper import pdf
from vollib.black_scholes_merton import d1,d2, black_scholes_merton


# -----------------------------------------------------------------------------
# FUNCTIONS - ANALYTICAL GREEKS


def delta(flag, S, K, t, r, sigma, q):
    
    """Returns the Black-Scholes-Merton delta of an option.
    

    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float
    :param q: annualized continuous dividend yield
    :type q: float
    
    :returns:  float 

    """

    D1 = d1(S, K, t, r, sigma, q)

    if flag == 'p':
        return -numpy.exp(-q*t) * cnd(-D1)
    else:
        return numpy.exp(-q*t) * cnd(D1)


def theta(flag, S, K, t, r, sigma, q):

    """Returns the Black-Scholes-Merton theta of an option.


    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float
    :param q: annualized continuous dividend yield
    :type q: float

    :returns:  float 

    """


    D1 = d1(S, K, t, r, sigma, q)
    D2 = d2(S, K, t, r, sigma, q)

    first_term = (S * numpy.exp(-q*t) * pdf(D1) * sigma) / (2 * numpy.sqrt(t))

    if flag == 'c':

        second_term = -q * S * numpy.exp(-q*t) * cnd(D1)
        third_term = r * K * numpy.exp(-r*t) * cnd(D2)

        return - (first_term + second_term + third_term) / 365.0

    else:

        second_term = -q * S * numpy.exp(-q*t) * cnd(-D1)
        third_term = r * K * numpy.exp(-r*t) * cnd(-D2)

        return (-first_term + second_term + third_term) / 365.0



def gamma(flag, S, K, t, r, sigma, q):


    """Returns the Black-Scholes-Merton gamma of an option.


    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float
    :param q: annualized continuous dividend yield
    :type q: float

    :returns:  float 

    """

    D1 = d1(S, K, t, r, sigma, q)
    numerator = numpy.exp(-q*t) * pdf(D1)
    denominator = S * sigma * numpy.sqrt(t)

    return numerator / denominator


def vega(flag, S, K, t, r, sigma, q):

    """Returns the Black-Scholes-Merton vega of an option.


    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float
    :param q: annualized continuous dividend yield
    :type q: float

    :returns:  float 

    """

    D1 = d1(S, K, t, r, sigma, q)

    return S * numpy.exp(-q*t) * pdf(D1) * numpy.sqrt(t) * 0.01


def rho(flag, S, K, t, r, sigma, q):


    """Returns the Black-Scholes-Merton rho of an option.


    :param flag: 'c' or 'p' for call or put.
    :type flag: str
    :param S: underlying asset price
    :type S: float
    :param K: strike price
    :type K: float
    :param t: time to expiration in years
    :type t: float
    :param r: annual risk-free interest rate
    :type r: float
    :param sigma: volatility
    :type sigma: float
    :param q: annualized continuous dividend yield
    :type q: float

    :returns:  float 

    """


    D2 = d2(S, K, t, r, sigma, q)

    if flag == 'c':

        return t * K * numpy.exp(-r*t) * cnd(D2) * .01

    else:

        return -t * K * numpy.exp(-r*t) * cnd(-D2) * .01


# -----------------------------------------------------------------------------
# MAIN
if __name__=='__main__':  
    import doctest
    if not doctest.testmod().failed:
        print "Doctest passed"
        






#!/usr/bin/env python
# encoding: utf-8

from setuptools import setup, find_packages


setup(
    name='vollib',
    version='0.1.2',
    description='',
    url='http://vollib.org',
    download_url='git+https://github.com/vollib/vollib.git#egg=vollib',
    maintainer='vollib',
    maintainer_email='support@quantycarlo.com',
    license='MIT',
    install_requires = [
        'lets_be_rational',
        'simplejson',
        'matplotlib',
        'pandas',
        'scipy'
    ],
    packages=find_packages()
)
